@@ -5,6 +5,7 @@ import logging
 from threading import Thread
 import requests
 import websocket
+from bs4 import BeautifulSoup
 from senertec.energyUnit import energyUnit
 from senertec.lang import lang
 from senertec.canipError import canipError
@@ -22,7 +23,6 @@ class basesocketclient:
         self.__is_ws_connected__ = False
         self.__messages__ = [str()]
         self.__thread__ = None
-        self.__should_stop__ = False
         self.ws = None
         self.boards = [board()]
         """All available boards with datapoints which can be used in request function"""
@@ -45,13 +45,17 @@ class basesocketclient:
                             if point.enumName != None:
                                 for enum in self.__enumTranslations__:
                                     if point.enumName == enum["name"]:
-                                        value.dataValue = enum["translations"][f"{tempValue}"]
+                                        try:
+                                            value.dataValue = enum["translations"][f"{tempValue}"]
+                                        except KeyError:
+                                            self.logger.warning(
+                                                f"No enum translation found for datapoint '{point.friendlyName}'.")
+                                            value.dataValue = "Unknown"
                             elif point.gain != 0 and point.gain != 1:
                                 value.dataValue = tempValue * \
                                     point.gain
                             else:
                                 value.dataValue = tempValue
-                            value.dataUnit = point.unit
                             if point.unit != None:
                                 value.dataUnit = point.unit
                             else:
@@ -62,19 +66,17 @@ class basesocketclient:
         elif action == "HkaStore" and data["updateType"] == "remove":
             sn = j["sn"]
             self.logger.info(
-                f"Unit with serial {sn} got disconnected. Started reconnect..")
-            self.connectUnit(sn)
+                f"Unit with serial {sn} got disconnected.")
         else:
             self.logger.debug(f"Received new websocket message {action}.")
             self.__messages__.append(message)
 
     def __on_error__(self, ws, error):
         self.logger.error(f"error : {error}")
-        self.__is_ws_connected__ = False
 
     def __on_close__(self, ws, close_status_code, close_msg):
         self.logger.info(
-            f"Connection closed to Senertec with code {close_status_code}")
+            f"Senertec Websocket closed with code {close_status_code}")
         self.__is_ws_connected__ = False
 
     def __on_open__(self, ws):
@@ -83,9 +85,9 @@ class basesocketclient:
 
     def __create_websocket__(self):
         cookies = self.__getCookies__(
-            self.__clientCookie__, "dachsconnect.senertec.com")
+            self.__session__.cookies, "dachsconnect.senertec.com")
         self.logger.debug("Creating websocket connection..")
-        self.ws = websocket.WebSocketApp("wss://dachsconnect.senertec.com/dachsportal2/ws",
+        self.ws = websocket.WebSocketApp("wss://dachsconnect.senertec.com/ws",
                                          on_message=self.__on_message__,
                                          on_error=self.__on_error__,
                                          on_close=self.__on_close__,
@@ -121,21 +123,21 @@ class senertec(basesocketclient):
             format='py-senertec: %(asctime)s %(levelname)-8s %(message)s',
             datefmt='%Y-%m-%d %H:%M:%S'
         )
-        self.AUTHENTICATION_HOST = "https://dachsconnect.senertec.com/dachsportal2"
+        self.AUTHENTICATION_HOST = "https://dachsconnect.senertec.com"
         self.email = email
+        self.__session__ = None
         self.language = language
         self.password = password
         self.level = level
         self.__supportedItems__ = dataNames
         self.__enums__ = []
-        self.__clientCookie__ = None
         self.__metaDataPoints__ = []
         self.__metaDataTranslations__ = []
         self.__enumTranslations__ = []
         self.__errorTranslations__ = []
         self.__connectedUnit__ = []
         self.messagecallback = (canipValue())
-        """Set your callback function to get the data values. Function has to be overloaded with data type ``canipvalue``"""
+        """Set your callback function to get the data values. Function has to be overloaded with data type ``canipValue``"""
         self.logger = logging.getLogger(__name__)
         self.logger.setLevel(level)
 
@@ -151,8 +153,8 @@ class senertec(basesocketclient):
 
     def __post__(self, urlPath: str, payload: str):
         url = self.AUTHENTICATION_HOST + urlPath
-        response = requests.post(
-            url, data=payload, headers=self.__create_headers__(), cookies=self.__clientCookie__)
+        response = self.__session__.post(
+            url, data=payload, headers=self.__create_headers__())
         if(response.status_code >= 400 and response.status_code <= 599):
             logger.error("Error in post request by function: " + inspect.stack()
                          [1].function + " HTTP response: " + response.text)
@@ -160,17 +162,18 @@ class senertec(basesocketclient):
 
     def __get__(self, urlPath: str):
         url = self.AUTHENTICATION_HOST + urlPath
-        response = requests.get(
-            url, headers=self.__create_headers__(), cookies=self.__clientCookie__)
+        response = self.__session__.get(
+            url, headers=self.__create_headers__())
         if(response.status_code >= 400 and response.status_code <= 599):
-            logger.error("Error in get request by function: " + inspect.stack()
-                         [1].function + " HTTP response: " + response.text)
+            logger.error("Error in get request by function: {" + inspect.stack()
+                         [1].function + "} HTTP response: " + response.text)
         return response
 
     def __parsedatapoints__(self):
         self.logger.debug("Starting to parse datapoints..")
         metaData = self.__metaDataPoints__
         blist = []
+        dataPointCount = 0
         for a in metaData:
             for element in self.__supportedItems__[self.__connectedUnit__["productGroup"]]:
                 if metaData[a]["friendlyName"] == element:
@@ -186,6 +189,7 @@ class senertec(basesocketclient):
                     datap.unit = metaData[a]["unit"]
                     datap.gain = metaData[a]["gain"]
                     datap.enumName = metaData[a]["enumName"]
+                    dataPointCount += 1
                     # avoid doubled board entries
                     if not any(x for x in blist if x.boardName == boardname):
                         b = board()
@@ -197,25 +201,50 @@ class senertec(basesocketclient):
                             if b.boardName == boardname:
                                 b.datapoints.append(datap)
         self.boards = blist
-        self.logger.debug(f"Parsing datapoints finished, found {len(blist)}.")
+        self.logger.debug(
+            f"Parsing datapoints finished, found {len(blist)} boards with {dataPointCount} datapoints.")
 
     def login(self):
         """
-        Authenticate and get cookie.
+        Login.
 
         This function needs to be called first.
 
         Returns True on success, False on failure.
         """
         self.logger.info("Logging in..")
-        response = self.__post__("/rest/info/login", json.dumps(
-            {"user": self.email, "password": self.password}))
-        if response.status_code == 200:
-            self.__clientCookie__ = response.cookies
-            self.logger.debug("Login was successful.")
-            return True
-        else:
+        self.__session__ = requests.Session()
+        loginSSOResponse = self.__session__.get(
+            "https://dachsconnect.senertec.com/rest/saml2/login")
+
+        authState = loginSSOResponse.url.split("loginuserpass.php?")[1]
+        head = {"Content-Type": "application/x-www-form-urlencoded"}
+        userData = f"username={self.email}&password={self.password}&{authState}"
+        # submit credentials
+        loginResponse = self.__session__.post("https://sso-portal.senertec.com/simplesaml/module.php/core/loginuserpass.php?",
+                                              data=userData, headers=head)
+
+        # filter out samlresponse and relaystate for ACS request
+        soup = BeautifulSoup(loginResponse.text, features="html.parser")
+        try:
+            samlResponse = soup.findAll(
+                "input", {"name": "SAMLResponse"})[0]["value"]
+            relayState = soup.findAll(
+                "input", {"name": "RelayState"})[0]["value"]
+        except IndexError:
+            self.logger.error("Login failed, username or password wrong.")
             return False
+        acsData = {'SAMLResponse': {samlResponse}, 'RelayState': {relayState}}
+
+        # do assertion consumer service request with received saml response
+        acs = self.__session__.post(
+            "https://dachsconnect.senertec.com/rest/saml2/acs", data=acsData, headers=head)
+
+        if acs.history[0].status_code != 302:
+            self.logger.error("Login failed at ACS request, got no redirect.")
+            return False
+        self.logger.info("Login was successful.")
+        return True
 
     def logout(self):
         """
@@ -226,6 +255,7 @@ class senertec(basesocketclient):
         self.logger.info("Logging out..")
         response = self.__get__("/logout")
         self.ws.close()
+        self.__session__.close()
         if response.status_code == 200:
             self.logger.debug("Logout was successful.")
             return True
@@ -349,11 +379,11 @@ class senertec(basesocketclient):
             lst.append(b["name"])
         return lst
 
-    def getErrors(self, onlyCurrentErrors: bool = True):
+    def getErrors(self, onlyCurrentErrors: bool = True) -> list[canipError]:
         """Get an error history of the connected unit. 
 
         This gets loaded when a unit gets connected."""
-        lst = [canipError]
+        lst = []
         for b in self.__connectedUnit__["boards"]:
             for e in b["canipErrors"]:
                 if onlyCurrentErrors and not e["currentError"]:
